//=================================================================================
// GCam Rec 1.0
//
// A camera work editor script by Gigan
//
// This script needs "gcamrec_component.hpp".
// Include "gcamrec_component.hpp" to "description.ext".
//
// Execute GCam Rec
// hd = [ Unit ] execVM "gcam_rec.sqf";
//
//=================================================================================
// KEYBINDS
//
// W,A,S,D : Move Horizontal
// Q,Z : Up/Down
// 2,X : Move Straight
// Mouse Move : Turning
// Mouse L-Drag (editor opend) : Turning
// Mouse Wheel : Zooming
// Mouse R-Click : To Default Zoom
// F : Brake
// C : Stop Zooming
// R : Start/Stop Recording
// O : Open Scene Editor
// H : Help
// Space : Quit GCam Rec
//
//=================================================================================
// CONFIG

// Move Speed ( Recommended 0.0-1.0 )
#define CFMOVE 0.5

// Turn Speed ( Recommended 0.0-1.0 )
#define CFTRK 0.5

// Zoom Speed ( Recommended 0.0-1.0 )
#define CFWHEEL 0.3

// Brake Strength Ratio ( Default: x1.0 )
#define CFBRAKE 1.0

// Script Processing Loop Interval ( Default: 0.010 seconds )
// If the value is smaller, recorded smoothly, data size increases, processing load high.
// The recorded scene fps should be adjusted to equal the cutscene playing fps.
#define INTERVAL 0.01

//=================================================================================
// CONFIG ADVANCED

// Initial camera parameter.
#define INITCAMDISY 7.5
#define INITCAMDISZ 2.5
#define INITCAMAGL -8
#define INITCAMZOOM 0.7

// Dialog text color.
#define COLORTEXT [ 0.8784, 0.8471, 0.651, 1.0 ]

// Dialog button text color.
#define COLORBUTTONTEXTOFF [ 0.874, 0.843, 0.647, 1.0 ]
#define COLORBUTTONTEXTON [ 1.00, 0.00, 0.00, 1.0 ]

//=================================================================================


#include "gcam_rec_savedata.hpp"

_recdata_array = [];
_recdatalabel_array = [];
if( !(isnil "_savedata") ) then
{
	_recdata_array = _savedata select 0;
	_recdatalabel_array = _savedata select 1;
};


GCam_KeyDown = [controlNull,-1,false,false,false];
GCam_KeyUp = [controlNull,-1,false,false,false];
GCam_MouseBD = [controlNull,-1,0.5,0.5,false,false,false];
GCam_MouseBU = [controlNull,-1,0.5,0.5,false,false,false];
GCam_MouseMove = [controlNull,0.0,0.0];
GCam_MouseWheel = [controlNull,0];

_loop = true;
_obj = objnull;
_cam = objnull;
_pos = [0,0,0];
_dir = 0;
_dive = 0;
_zoom = INITCAMZOOM;
_accdir = 0;
_accdive = 0;
_acczoom = 0;
_accx = 0;
_accy = 0;
_accz = 0;
_objpos = getposATL player;
_campos = [0,0,0];
_campos_rel = [0,0,0];
_cfzoom = sin((_zoom / 1.8) * 90);
_cnt = 0;
_cnt2 = 0;
_helptext = "-= GCam Rec =-\n\nW,A,S,D : Move Horizontal\nQ,Z : Up/Down\n2,X : Move Straight\nMouse Move : Turning\nMouse L-Drag (editor opend) : Turning\nMouse Wheel : Zooming\nMouse R-Click : To Default Zoom\nF : Brake\nC : Stop Zooming\nR : Start/Stop Recording\nO : Open Scene Editor\nH : Help\nSpace : Quit GCam Rec\n\nVer. 1.0";
_quit = false;
_quitchk = false;
_outstr = "";
_isallcopy = false;
_selnum = 0;
_isframes = true;

_keyd = 0;
_keyu = 0;
_wheel = 0;
_mousemove = false;
_mousepos = [0,0];
_mousebd = -1;
_mousebu = -1;

_rec = false;
_reccnt = 1;
_recdata_prv = [];
_recdata_tmp = [];
_recdata_array_tmp = [];
_recdatalabel_tmp = [];
_recdatalabel_array_tmp = [];

_editorshow = false;
_helpshow = false;

_ed_listselnum = -1;
_ed_recdata = [];
_ed_sliderpos = -1;
_ed_sliderpos_last = -1;
_ed_reso = 0.0;
_ed_framenum = 0;
_ed_scenenum = -1;
_ed_tickstart = 0.0;
_ed_itemadded = false;
_ed_mouse_onctrl = false;
_ed_button_id = "";
_ed_framenum_sels = -1;
_ed_framenum_sele = -1;
_ed_play = false;
_ed_playall = false;
_ed_hst_array = [];
_ed_text = [];


_Convert =
{
	_recdata_tmp = _recdata_array select 0;
	
	_outstr = "";
	_outstr = _outstr +
"//==================================================================================================
// GCam Rec 1.0 Output Script
//
// This script offers the camera work in cutscene for the mission editing.
//
// hd = [ unit ] execVM ""this_script.sqf"";
// or
// hd = [ x-pos, y-pos, z-pos ] execVM ""this_script.sqf"";
//
// The unit or coordinates can be specified for the arguments.
// The camera moves relatively from the origin of the coordinate axes specified for the arguments.
//
//==================================================================================================
// CONFIG

";
	_outstr = _outstr + format["// Update interval ( default: %1 seconds )", INTERVAL];
	
	_outstr = _outstr +
"
// If the difference is found to recording speed and this camera playing speed, the value is adjusted.
// In many cases, this camera moves a little quickly than the recording camera.
";
	
	_outstr = _outstr + format["#define INTERVAL %1", INTERVAL];
	
	_outstr = _outstr +
"

//==================================================================================================


//----------------------------------------
// Processing preparation Start
//----------------------------------------
";
	_outstr = _outstr + "private [""_obj"",""_objpos"",""_cam"",""_campos"",""_recdata"",""_mode""];";
	_outstr = _outstr +
"
if (count _this == 1) then
{
	_obj = _this select 0;
	_objpos = getposASL _obj;
	_mode = true;
};
if (count _this == 3) then
{
	_objpos = [ _this select 0, _this select 1, _this select 2 ];
	_mode = false;
};
//----------------------------------------
// Processing preparation End
//----------------------------------------


//----------------------------------------
// Creating Camera Object Start
//----------------------------------------
_cam = ""camera"" camCreate [0,0,0];
";
	
	_outstr = _outstr + format[ "_cam setPosASL [(_objpos select 0)+%1,(_objpos select 1)+%2,(_objpos select 2)+%3];", (_recdata_tmp select 0) select 0, (_recdata_tmp select 0) select 1, (_recdata_tmp select 0) select 2 ];
	_outstr = _outstr +
"
";
	
	_outstr = _outstr + format[ "_cam camSetTarget [%1, %2, %3];", ((_recdata_tmp select 0) select 0) + (cos((_recdata_tmp select 0) select 3)) * (cos((_recdata_tmp select 0) select 4)) * 100000, ((_recdata_tmp select 0) select 1) + (sin((_recdata_tmp select 0) select 3)) * (cos((_recdata_tmp select 0) select 4)) * 100000, ((_recdata_tmp select 0) select 2) + (sin((_recdata_tmp select 0) select 4)) * 100000 ];
	_outstr = _outstr +
"
";
	
	_outstr = _outstr +
"_cam cameraEffect [""Internal"", ""Back""];
";
	
	_outstr = _outstr + format[ "_cam camSetFov %1;", ((_recdata_tmp select 0) select 5) ];
	_outstr = _outstr +
"
";
	
	_outstr = _outstr +
"_cam camCommit 0;
sleep 0.01;
showCinemaBorder false;
//----------------------------------------
// Creating Camera Object End
//----------------------------------------

";
	if (_isallcopy) then
	{ _selnum = 0 }
	else
	{ _selnum = _ed_scenenum };
	
	_cnt2 = 0;
	{
		if ( _cnt2 == _selnum ) then
		{
			_outstr = _outstr +
"
//----------------------------------------
";
			_outstr = _outstr + format[ "// ""%1"" Start", (_recdatalabel_array select _cnt2) select 0 ];
			_outstr = _outstr +
"
";
			_outstr = _outstr + format[ "// %1 frames recorded in %2 seconds", (_recdatalabel_array select _cnt2) select 1, (round(((_recdatalabel_array select _cnt2) select 2)*10))*0.1 ];
			_outstr = _outstr +
"
//----------------------------------------
_recdata = [";
			
			
			_cnt = 0;
			{
				if (_cnt > 0) then { _outstr = _outstr + "," };
				_outstr = _outstr + format[ "[%1,%2,%3,%4,%5,%6]", _x select 0, _x select 1, _x select 2, _x select 3, _x select 4, _x select 5 ];
				_cnt = _cnt + 1;
			} foreach _x;
			
			
			_outstr = _outstr + "];";
			
			_outstr = _outstr +
"
{
	_cam camSetFov (_x select 5);
	if (_mode) then { _objpos = getposasl _obj };
	_campos = [ (_objpos select 0) + (_x select 0), (_objpos select 1) + (_x select 1), (_objpos select 2) + (_x select 2) ];
	_cam setPosASL _campos;
	_cam camSetTarget [ (_campos select 0) + (cos(_x select 3)) * (cos(_x select 4)) * 100000, (_campos select 1) + (sin(_x select 3)) * (cos(_x select 4)) * 100000, (_campos select 2) + (sin(_x select 4)) * 100000];
	_cam camCommit 0;
	sleep INTERVAL;
} foreach _recdata;
//----------------------------------------
";
			_outstr = _outstr + format[ "// ""%1"" End", (_recdatalabel_array select _cnt2) select 0 ];
			_outstr = _outstr +
"
//----------------------------------------

";
		};
		if (_isallcopy) then { _selnum = _selnum + 1 };
		_cnt2 = _cnt2 + 1;
	} foreach _recdata_array;
	
	_outstr = _outstr +
"
//----------------------------------------
// Destroying Camera Object Start
//----------------------------------------
_cam cameraEffect [""Terminate"", ""BACK""];
camDestroy _cam;
//----------------------------------------
// Destroying Camera Object End
//----------------------------------------
";
	
	copyToClipboard _outstr;
};


_OpenEditor =
{
	_helpshow = false;
	call _CloseHelp;
	
	_accx = 0.0;
	_accy = 0.0;
	_accz = 0.0;
	_accdir = 0.0;
	_accdive = 0.0;
	_acczoom = 0.0;
	
	_ed_scenenum = -1;
	_ed_recdata = [];
	_ed_play = false;
	_ed_playall = false;
	
	call _CloseHelp;
	
	createDialog "GCamRec_Dialog_Editor";
	waituntil{ctrlShown(findDisplay 5200 displayCtrl 10)};
	setmouseposition [0.50,0.65];
	
	(findDisplay 5200 displayCtrl 10) ctrlSetTextColor [ 1.0, 1.0, 1.0, 1.0 ];
	
	(findDisplay 5200 displayCtrl 14) ctrlSetTextColor COLORBUTTONTEXTOFF;
	
	if (_rec) then
	{ (findDisplay 5200 displayCtrl 20) ctrlSetTextColor COLORBUTTONTEXTON }
	else
	{ (findDisplay 5200 displayCtrl 20) ctrlSetTextColor COLORBUTTONTEXTOFF };
	(findDisplay 5200 displayCtrl 21) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 22) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 23) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 24) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 25) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 26) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 27) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 28) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 29) ctrlSetTextColor COLORBUTTONTEXTOFF;
	
	(findDisplay 5200 displayCtrl 40) ctrlSetTextColor COLORTEXT;
	(findDisplay 5200 displayCtrl 41) ctrlSetTextColor COLORTEXT;
	(findDisplay 5200 displayCtrl 42) ctrlSetTextColor COLORTEXT;
	(findDisplay 5200 displayCtrl 43) ctrlSetTextColor COLORTEXT;
	(findDisplay 5200 displayCtrl 44) ctrlSetTextColor COLORTEXT;
	
	(findDisplay 5200 displayCtrl 40) ctrlSetText "-";
	(findDisplay 5200 displayCtrl 41) ctrlSetText "";
	(findDisplay 5200 displayCtrl 42) ctrlSetText "";
	(findDisplay 5200 displayCtrl 43) ctrlSetText "";
	(findDisplay 5200 displayCtrl 44) ctrlSetText "";
	
	sliderSetRange [50, 0.00, 1.00];
	sliderSetspeed [50, 1.00, 0.00];
	
	(findDisplay 5200 displayCtrl 60) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 61) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 62) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 63) ctrlSetTextColor COLORBUTTONTEXTOFF;
	(findDisplay 5200 displayCtrl 64) ctrlSetTextColor COLORBUTTONTEXTOFF;
	
	call _ListRefresh;
	
	ctrlSetFocus (findDisplay 5200 displayCtrl 50);
};


_CloseEditor =
{
	_editorshow = false;
	
	_ed_play = false;
	_ed_playall = false;
	
	closedialog 5200;
	waituntil{!(ctrlShown(findDisplay 5200 displayCtrl 10))};
	
	_mousemove = false;
	GCam_KeyDown set [1,-1];
	GCam_KeyUp set [1,-1];
	GCam_MouseBD set [1,-1];
	GCam_MouseBU set [1,-1];
	GCamRec_MouseMoveCtrl set [3,false];
};


_OpenHelp =
{
	_editorshow = false;
	call _CloseEditor;
	
	_accx = 0.0;
	_accy = 0.0;
	_accz = 0.0;
	_accdir = 0.0;
	_accdive = 0.0;
	_acczoom = 0.0;
	
	createDialog "GCamRec_Dialog_Help";
	waituntil{ctrlShown(findDisplay 5202 displayCtrl 10)};
	setmouseposition [0.50,0.65];
	
	(findDisplay 5202 displayCtrl 10) ctrlSetTextColor COLORTEXT;
	(findDisplay 5202 displayCtrl 10) ctrlSetText _helptext;
	(findDisplay 5202 displayCtrl 20) ctrlSetTextColor COLORTEXT;
	
	ctrlSetFocus (findDisplay 5202 displayCtrl 20);
};


_CloseHelp =
{
	_helpshow = false;
	
	closedialog 5202;
	waituntil{!(ctrlShown(findDisplay 5202 displayCtrl 10))};
	
	GCam_KeyDown set [1,-1];
	GCam_KeyUp set [1,-1];
	GCam_MouseBD set [1,-1];
	GCam_MouseBU set [1,-1];
};


_StopPlaying =
{
	if (_ed_play) then
	{
		_ed_play = false;
		(findDisplay 5200 displayCtrl 21) ctrlSetTextColor COLORBUTTONTEXTOFF;
	};
	if (_ed_playall) then
	{
		_ed_playall = false;
		(findDisplay 5200 displayCtrl 22) ctrlSetTextColor COLORBUTTONTEXTOFF;
	};
};


_ListRefresh =
{
	lbClear 10;
	
	if ( (count _recdata_array) > 0 ) then
	{
		_cnt = 0;
		{
			lbAdd[ 10, format[ "%1 : %2f %3s %4fps", (_recdatalabel_array select _cnt) select 0, (_recdatalabel_array select _cnt) select 1,(round(((_recdatalabel_array select _cnt) select 2)*10))*0.1, (round((((_recdatalabel_array select _cnt) select 1)/((_recdatalabel_array select _cnt) select 2))*10))*0.1 ] ];
			_cnt = _cnt + 1;
		} foreach _recdata_array;
	};
};


_SeekbarRefresh =
{
	if ( (count _ed_recdata) == 1 ) then
	{
		_isframes = false;
		_ed_reso = 1.0;
	}
	else
	{
		_isframes = true;
		_ed_reso = 1.0 / ((count _ed_recdata)-1);
	};
	sliderSetspeed [50, _ed_reso, 0.0];
	sliderSetPosition [50, _ed_reso * _ed_framenum];
	
	(findDisplay 5200 displayCtrl 41) ctrlSetText "";
	(findDisplay 5200 displayCtrl 42) ctrlSetText "";
	(findDisplay 5200 displayCtrl 43) ctrlSetText format["F%1",_ed_framenum + 1];
	(findDisplay 5200 displayCtrl 44) ctrlSetText format["F%1",count _ed_recdata];
	
	ctrlSetFocus (findDisplay 5200 displayCtrl 50);
	
	_ed_framenum_sels = -1;
	_ed_framenum_sele = -1;
};


_PosStopRefresh =
{
	_accx = 0.0;
	_accy = 0.0;
	_accz = 0.0;
	_accdir = 0.0;
	_accdive = 0.0;
	_acczoom = 0.0;
	
	_campos_rel = [ (_ed_recdata select _ed_framenum) select 0, (_ed_recdata select _ed_framenum) select 1, (_ed_recdata select _ed_framenum) select 2 ];
	_dir = (_ed_recdata select _ed_framenum) select 3;
	_dive = (_ed_recdata select _ed_framenum) select 4;
	_zoom = (_ed_recdata select _ed_framenum) select 5;
	_cam camSetFov _zoom;
	_cfzoom = sin ((_zoom / 1.8) * 90);
};


_SceneDelete =
{
	_cnt = 0;
	_recdata_array_tmp = [];
	_recdatalabel_array_tmp = [];
	{
		if ( _ed_scenenum != _cnt ) then
		{
			_recdata_array_tmp = _recdata_array_tmp + [(_recdata_array select _cnt)];
			_recdatalabel_array_tmp = _recdatalabel_array_tmp + [(_recdatalabel_array select _cnt)];
		};
		_cnt = _cnt + 1;
	} foreach _recdata_array;
	_recdata_array = _recdata_array_tmp;
	_recdatalabel_array = _recdatalabel_array_tmp;
	
	if ((count _recdata_array) == 0) then { _ed_scenenum = -1 };
};


_SceneUp =
{
	_cnt = 0;
	_recdata_tmp = _recdata_array select (_ed_scenenum-1);
	_recdatalabel_tmp = _recdatalabel_array select (_ed_scenenum-1);
	_recdata_array_tmp = [];
	_recdatalabel_array_tmp = [];
	{
		switch (true) do
		{
			case ( (_ed_scenenum-1) == _cnt ):
			{
				_recdata_array_tmp = _recdata_array_tmp + [_recdata_array select (_cnt+1)];
				_recdatalabel_array_tmp = _recdatalabel_array_tmp + [_recdatalabel_array select (_cnt+1)];
			};
			case ( _ed_scenenum == _cnt ):
			{
				_recdata_array_tmp = _recdata_array_tmp + [_recdata_tmp];
				_recdatalabel_array_tmp = _recdatalabel_array_tmp + [_recdatalabel_tmp];
			};
			default
			{
				_recdata_array_tmp = _recdata_array_tmp + [_recdata_array select _cnt];
				_recdatalabel_array_tmp = _recdatalabel_array_tmp + [_recdatalabel_array select _cnt];
			};
		};
		_cnt = _cnt + 1;
	} foreach _recdata_array;
	_recdata_array = _recdata_array_tmp;
	_recdatalabel_array = _recdatalabel_array_tmp;
	
	_ed_scenenum = _ed_scenenum - 1;
};


_SceneDown =
{
	_cnt = 0;
	_recdata_tmp = _recdata_array select _ed_scenenum;
	_recdatalabel_tmp = _recdatalabel_array select _ed_scenenum;
	_recdata_array_tmp = [];
	_recdatalabel_array_tmp = [];
	{
		switch (true) do
		{
			case ( _ed_scenenum == _cnt ):
			{
				_recdata_array_tmp = _recdata_array_tmp + [_recdata_array select (_cnt+1)];
				_recdatalabel_array_tmp = _recdatalabel_array_tmp + [_recdatalabel_array select (_cnt+1)];
			};
			case ( (_ed_scenenum+1) == _cnt ):
			{
				_recdata_array_tmp = _recdata_array_tmp + [_recdata_tmp];
				_recdatalabel_array_tmp = _recdatalabel_array_tmp + [_recdatalabel_tmp];
			};
			default
			{
				_recdata_array_tmp = _recdata_array_tmp + [_recdata_array select _cnt];
				_recdatalabel_array_tmp = _recdatalabel_array_tmp + [_recdatalabel_array select _cnt];
			};
		};
		_cnt = _cnt + 1;
	} foreach _recdata_array;
	_recdata_array = _recdata_array_tmp;
	_recdatalabel_array = _recdatalabel_array_tmp;
	
	_ed_scenenum = _ed_scenenum + 1;
};


_SceneEditCut =
{
	_cnt = 0;
	_recdata_tmp = [];
	{
		if ( (_cnt < _ed_framenum_sels) or ( _ed_framenum_sele < _cnt ) ) then
		{
			_recdata_tmp = _recdata_tmp + [_ed_recdata select _cnt];
		};
		_cnt = _cnt + 1;
	} foreach _ed_recdata;
	_ed_recdata = _recdata_tmp;
	
	_cnt = 0;
	_recdata_array_tmp = [];
	_recdatalabel_array_tmp = [];
	{
		if (_ed_scenenum == _cnt) then
		{
			_recdata_array_tmp = _recdata_array_tmp + [_ed_recdata];
			_recdatalabel_array_tmp = _recdatalabel_array_tmp + [[ (_recdatalabel_array select _ed_scenenum) select 0, count _ed_recdata, ((_recdatalabel_array select _ed_scenenum) select 2) * ((count _ed_recdata)/((_recdatalabel_array select _ed_scenenum) select 1)) ]];
		}
		else
		{
			_recdata_array_tmp = _recdata_array_tmp + [_recdata_array select _cnt];
			_recdatalabel_array_tmp = _recdatalabel_array_tmp + [_recdatalabel_array select _cnt];
		};
		_cnt = _cnt + 1;
	}
	foreach _recdata_array;
	_recdata_array = _recdata_array_tmp;
	_recdatalabel_array = _recdatalabel_array_tmp;
	
	if (_ed_framenum_sels == 0) then
	{ _ed_framenum = 0}
	else
	{ _ed_framenum = _ed_framenum_sels - 1 };
	
	_ed_framenum_sels = -1;
	_ed_framenum_sele = -1;
};


_SceneEditNew =
{
	_cnt = 0;
	_recdata_tmp = [];
	
	{
		if ( (_ed_framenum_sels <= _cnt) and (_cnt <= _ed_framenum_sele) ) then
		{
			_recdata_tmp = _recdata_tmp + [(_ed_recdata select _cnt)];
		};
		_cnt = _cnt + 1;
	} foreach _ed_recdata;
	
	_recdata_array = _recdata_array + [_recdata_tmp];
	
	_recdatalabel_tmp = [ format["Scene %1",_reccnt], count _recdata_tmp, ((_recdatalabel_array select _ed_scenenum) select 2) * ((count _recdata_tmp)/((_recdatalabel_array select _ed_scenenum) select 1)) ];
	_recdatalabel_array = _recdatalabel_array + [_recdatalabel_tmp];
	_reccnt = _reccnt + 1;
};


_HistoryAdd =
{
	_ed_hst_array = _ed_hst_array + [[_recdata_array, _recdatalabel_array, _ed_scenenum]];
};


_Undo =
{
	_recdata_array = (_ed_hst_array select ((count _ed_hst_array)-1)) select 0;
	_recdatalabel_array = (_ed_hst_array select ((count _ed_hst_array)-1)) select 1;
	_ed_scenenum = (_ed_hst_array select ((count _ed_hst_array)-1)) select 2;
	
	_ed_hst_array set [(count _ed_hst_array)-1, "nil"];
	_ed_hst_array = _ed_hst_array - ["nil"];
};


_Rename =
{
	_cnt = 0;
	_recdata_tmp = _recdata_array select _ed_scenenum;
	_recdatalabel_tmp = [ ctrlText 15, count _recdata_tmp, ((_recdatalabel_array select _ed_scenenum) select 2) * ((count _recdata_tmp)/((_recdatalabel_array select _ed_scenenum) select 1)) ];
	_recdatalabel_array_tmp = [];
	{
		if (_ed_scenenum == _cnt) then
		{
			_recdatalabel_array_tmp = _recdatalabel_array_tmp + [_recdatalabel_tmp];
		}
		else
		{
			_recdatalabel_array_tmp = _recdatalabel_array_tmp + [_x];
		};
		_cnt = _cnt + 1;
	} foreach _recdatalabel_array;
	_recdatalabel_array = _recdatalabel_array_tmp;
};


_Save =
{
	_outstr = "";
	_outstr = _outstr +
"// GCam Rec 1.0 Save Data
// When GCam Rec starts, it is load first.
";
	_outstr = _outstr + "_savedata=[[";
	
	_cnt2 = 0;
	{
		if (_cnt2 > 0) then { _outstr = _outstr + "," };
		_outstr = _outstr + "[";
		_cnt = 0;
		{
			if (_cnt > 0) then { _outstr = _outstr + "," };
			_outstr = _outstr + format[ "[%1,%2,%3,%4,%5,%6]", _x select 0, _x select 1, _x select 2, _x select 3, _x select 4, _x select 5 ];
			_cnt = _cnt + 1;
		} foreach _x;
		_outstr = _outstr + "]";
		_cnt2 = _cnt2 + 1;
	} foreach _recdata_array;
	
	_outstr = _outstr + "],";
	
	_outstr = _outstr + "[";
	_cnt = 0;
	{
		if (_cnt > 0) then { _outstr = _outstr + "," };
		_outstr = _outstr + format[ "[""%1"",%2,%3]", _x select 0, _x select 1, _x select 2 ];
		_cnt = _cnt + 1;
	} foreach _recdatalabel_array;
	
	_outstr = _outstr + "]];";
	
	copyToClipboard _outstr;
};


sleep 0.01;

_ehid_keydown = (findDisplay 46) displayAddEventHandler ["KeyDown", "GCam_KeyDown = _this"];
_ehid_keyup = (findDisplay 46) displayAddEventHandler ["KeyUp", "GCam_KeyUp = _this"];
_ehid_mousemove = (findDisplay 46)  displayAddEventHandler ["MouseMoving", "GCam_MouseMove = _this"];
_ehid_mousezchange = (findDisplay 46) displayAddEventHandler ["MouseZChanged", "GCam_MouseWheel = _this"];
_ehid_mousebd = (findDisplay 46) displayAddEventHandler ["MouseButtonDown", "GCam_MouseBD = _this"];
_ehid_mousebu = (findDisplay 46) displayAddEventHandler ["MouseButtonUp", "GCam_MouseBU = _this"];

_obj = vehicle (_this select 0);
if (isnil"_obj") then { _obj = player };
_dir = 90;
_dive = INITCAMAGL;
_campos_rel = [0,- INITCAMDISY,INITCAMDISZ];
_objpos = getposASL _obj;
_campos = [ (_objpos select 0) + (_campos_rel select 0), (_objpos select 1) + (_campos_rel select 1), (_objpos select 2) + (_campos_rel select 2)];

_cam = "camera" camCreate [0,0,0];
_cam setPosASL [_campos select 0, _campos select 1, _campos select 2];
_cam camSetTarget [_campos select 0, (_campos select 1) + 1, _campos select 2];
_cam cameraEffect ["Internal", "Back"];
_campos = [0,0,0];
_cam camSetFov _zoom;

sleep 0.01;
showCinemaBorder false;


while {_loop} do
{
	if (!(isnil "GCam_MouseBD")) then { _mousebd = GCam_MouseBD select 1 };
	if (!(isnil "GCam_MouseBU")) then { _mousebu = GCam_MouseBU select 1 };
	if ( (_mousebu == 0) or (_mousebu == 1 and _mousebd == 1) ) then
	{
		_mousemove = false;
		GCam_MouseBD set [1, -1];
		GCam_MouseBU set [1, -1];
	};
	
	if (!(isnil "GCamRec_MouseMoveCtrl")) then { _ed_mouse_onctrl = GCamRec_MouseMoveCtrl select 3 };
	if (_mousebd == 0 and _mousebu != 0 and !(_mousemove) and !(_ed_mouse_onctrl)) then
	{
		_mousemove = true;
		ctrlSetFocus (findDisplay 5200 displayCtrl 50);
	};
	if (_mousebd == 1 and _mousebu == 1) then
	{
		_acczoom = 0;
		_zoom = 0.7;
		_cam camSetFov _zoom;
		_cfzoom = sin ((_zoom / 1.8) * 90);
	};
	
	if (!(_editorshow)and!(_helpshow)) then
	{
		_mousepos = [ GCam_MouseMove select 1, GCam_MouseMove select 2 ];
		
		_accdir = _accdir + (_mousepos select 0) * -0.04 * CFTRK * _cfzoom / (cos abs(_dive / (1.0 + _cfzoom)));
		_accdive = _accdive + (_mousepos select 1) * -0.18 * CFTRK * _cfzoom;
		
		GCam_MouseMove = [controlNull,0.0,0.0];
	};
	
	_dir = _dir + _accdir;
	if (_dir >= 360.0) then { _dir = _dir - 360.0 };
	if (_dir < 0.0) then { _dir = _dir + 360.0 };
	
	_dive = _dive + _accdive;
	if (_dive > 89.9) then
	{
		_dive = 89.9;
		_accdive = 0.0;
	};
	if (_dive < -89.9) then
	{
		_dive = -89.9;
		_accdive = 0.0;
	};
	
	
	if (!(isnil "GCam_KeyDown")) then { _keyd = GCam_KeyDown select 1 };
	if (!(isnil "GCam_KeyUp")) then { _keyu = GCam_KeyUp select 1 };
	if (_keyd == _keyu) then
	{
		GCam_KeyDown set [1, -1];
		GCam_KeyUp set [1, -1];
	};
	
	if (_keyd == 17 and _keyu != 17) then
	{
		_accx = _accx * 0.995;
		_accy = _accy * 0.995;
		_accz = _accz * 0.995;
		_accx = _accx + 0.0075 * (cos _dir) * CFMOVE * _cfzoom;
		_accy = _accy + 0.0075 * (sin _dir) * CFMOVE * _cfzoom;
	};
	if (_keyd == 31 and _keyu != 31) then
	{
		_accx = _accx * 0.995;
		_accy = _accy * 0.995;
		_accz = _accz * 0.995;
		_accx = _accx - 0.0075 * (cos _dir) * CFMOVE * _cfzoom;
		_accy = _accy - 0.0075 * (sin _dir) * CFMOVE * _cfzoom;
	};
	if (_keyd == 30 and _keyu != 30) then
	{
		_accx = _accx * 0.995;
		_accy = _accy * 0.995;
		_accz = _accz * 0.995;
		_accx = _accx - 0.0075 * (sin _dir) * CFMOVE * _cfzoom;
		_accy = _accy + 0.0075 * (cos _dir) * CFMOVE * _cfzoom;
	};
	if (_keyd == 32 and _keyu != 32) then
	{
		_accx = _accx * 0.995;
		_accy = _accy * 0.995;
		_accz = _accz * 0.995;
		_accx = _accx + 0.0075 * (sin _dir) * CFMOVE * _cfzoom;
		_accy = _accy - 0.0075 * (cos _dir) * CFMOVE * _cfzoom;
	};
	if (_keyd == 16 and _keyu != 16) then
	{
		_accx = _accx * 0.995;
		_accy = _accy * 0.995;
		_accz = _accz * 0.995;
		_accz = _accz + 0.0075 * CFMOVE * _cfzoom;
	};
	if (_keyd == 44 and _keyu != 44) then
	{
		_accx = _accx * 0.995;
		_accy = _accy * 0.995;
		_accz = _accz * 0.995;
		_accz = _accz - 0.0075 * CFMOVE * _cfzoom;
	};
	if (_keyd == 3 and _keyu != 3) then
	{
		_accx = _accx * 0.995;
		_accy = _accy * 0.995;
		_accz = _accz * 0.995;
		_accx = _accx + 0.01 * CFMOVE * (cos _dir) * (cos _dive) * _cfzoom;
		_accy = _accy + 0.01 * CFMOVE * (sin _dir) * (cos _dive) * _cfzoom;
		_accz = _accz + 0.01 * CFMOVE * (sin _dive) * _cfzoom;
	};
	if (_keyd == 45 and _keyu != 45) then
	{
		_accx = _accx * 0.995;
		_accy = _accy * 0.995;
		_accz = _accz * 0.995;
		_accx = _accx - 0.01 * CFMOVE * (cos _dir) * (cos _dive) * _cfzoom;
		_accy = _accy - 0.01 * CFMOVE * (sin _dir) * (cos _dive) * _cfzoom;
		_accz = _accz - 0.01 * CFMOVE * (sin _dive) * _cfzoom;
	};
	if (_keyd == 33 and _keyu != 33) then
	{
		_accx = _accx * (1.0 - (0.05 * CFBRAKE));
		_accy = _accy * (1.0 - (0.05 * CFBRAKE));
		_accz = _accz * (1.0 - (0.05 * CFBRAKE));
		_accdir = _accdir * (1.0 - (0.05 * CFBRAKE));
		_accdive = _accdive * (1.0 - (0.05 * CFBRAKE));
	};
	if (_keyd == 46 and _keyu != 46) then
	{
		_acczoom = 0;
	};
	if (_keyd == 35 and _keyu == 35) then
	{
		if ( !(ctrlShown(findDisplay 5202 displayCtrl 10)) ) then { _helpshow = false; };
		_helpshow = !(_helpshow);
		
		if (_helpshow) then
		{ call _OpenHelp }
		else
		{ call _CloseHelp };
	};
	if (_helpshow) then
	{
		if (GCamRec_ButtonPushedId == "helpclose") then
		{
			_helpshow = false;
			call _CloseHelp;
			
			GCamRec_ButtonPushedId = "";
		};
	};
	if (_keyd == 19 and _keyu == 19) then
	{
		_rec = !(_rec);
		
		if (_rec) then
		{
			if (_editorshow) then
			{
				(findDisplay 5200 displayCtrl 20) ctrlSetTextColor COLORBUTTONTEXTON;
			};
			
			_recdata_prv = [];
			_ed_tickstart = diag_ticktime;
			
			titletext["Recording Started","plain",0.15];
		}
		else
		{
			_accx = 0;
			_accy = 0;
			_accz = 0;
			_accdir = 0;
			_accdive = 0;
			_acczoom = 0.0;
			
			call _HistoryAdd;
			
			_recdata_array = _recdata_array + [_recdata_prv];
			_recdatalabel_array = _recdatalabel_array + [[ format["Scene %1",_reccnt], count _recdata_prv, (diag_ticktime - _ed_tickstart) ]];
			_reccnt = _reccnt + 1;
			
			if (_editorshow) then
			{
				(findDisplay 5200 displayCtrl 20) ctrlSetTextColor COLORBUTTONTEXTOFF;
				call _ListRefresh;
				lbSetCurSel [10, (count _recdata_array) - 1];
				_ed_itemadded = true;
			};
			
			titletext["Recording Finished","plain",0.3];
		};
	};
	if (_rec) then
	{
		_recdata_prv = _recdata_prv + [[_campos_rel select 0, _campos_rel select 1, _campos_rel select 2, _dir, _dive, _zoom]];
	};
	if (_keyd == 24 and _keyu == 24) then
	{
		_editorshow = !(_editorshow);
		
		if (_editorshow) then
		{ call _OpenEditor }
		else
		{ call _CloseEditor };
	};
	
	
	_wheel = GCam_MouseWheel select 1;
	if (_wheel > 0.0001 or _wheel < -0.0001) then
	{
		_acczoom = _acczoom - _wheel * 0.004 * CFWHEEL * sin((_zoom / 2.0) * 90);
		GCam_MouseWheel set [1, 0];
	};
	if ((abs _acczoom) > 0.00001) then
	{
		_zoom = _zoom + _acczoom;
		if (_zoom < 0.01) then
		{
			_zoom = 0.01;
			_acczoom = 0;
		};
		if (_zoom > 2.0) then
		{
			_zoom = 2.0;
			_acczoom = 0;
		};
		_cam camSetFov _zoom;
		_cfzoom = sin ((_zoom / 1.8) * 90);
	};
	
	
	if (_keyd == 57 or !(alive player)) then
	{
		if (_keyd == 57) then
		{
			call _CloseEditor;
			call _CloseHelp;
			
			createDialog "GCamRec_Dialog_Quit";
			waituntil{ctrlShown(findDisplay 5201 displayCtrl 10)};
			setmouseposition [0.500,0.575];
			
			(findDisplay 5201 displayCtrl 10) ctrlSetTextColor COLORTEXT;
			(findDisplay 5201 displayCtrl 20) ctrlSetTextColor COLORTEXT;
			(findDisplay 5201 displayCtrl 21) ctrlSetTextColor COLORTEXT;
			
			ctrlSetFocus (findDisplay 5201 displayCtrl 21);
		};
		
		_quit = false;
		_quitchk = true;
		while {_quitchk} do
		{
			if ( !(ctrlShown(findDisplay 5201 displayCtrl 10)) or GCamRec_ButtonPushedQuitCancel ) then
			{
				_quitchk = false;
				GCamRec_ButtonPushedQuitCancel = false;
			};
			if ( GCamRec_ButtonPushedQuitOk ) then
			{
				_quit = true;
				_quitchk = false;
				GCamRec_ButtonPushedQuitOk = false;
			};
			sleep INTERVAL;
		};
		
		if (_quit) then
		{
			(findDisplay 46) displayRemoveEventHandler ["KeyDown", _ehid_keydown];
			(findDisplay 46) displayRemoveEventHandler ["KeyUp", _ehid_keyup];
			(findDisplay 46) displayRemoveEventHandler ["MouseMoving", _ehid_mousemove];
			(findDisplay 46) displayRemoveEventHandler ["MouseZChanged", _ehid_mousezchange];
			(findDisplay 46) displayRemoveEventHandler ["MouseButtonDown", _ehid_mousebd];
			(findDisplay 46) displayRemoveEventHandler ["MouseButtonUp", _ehid_mousebu];
			
			_cam cameraEffect ["Terminate", "BACK"];
			camDestroy _cam;
			_loop = false;
			
			titletext["","plain",0.0];
		};
		
		closedialog 5201;
		GCam_KeyDown set [1, -1];
		GCam_KeyUp set [1, -1];
		GCam_MouseBD set [1,-1];
		GCam_MouseBU set [1,-1];
	};
	
	if (_editorshow) then
	{
		if (_mousemove) then
		{
			_mousepos = [ GCam_MouseMove select 1, GCam_MouseMove select 2 ];
			
			_accdir = _accdir + (_mousepos select 0) * -0.06 * CFTRK * _cfzoom / (cos abs(_dive / (1.0 + _cfzoom)));
			_accdive = _accdive + (_mousepos select 1) * -0.27 * CFTRK * _cfzoom;
			
			GCam_MouseMove = [controlNull,0.0,0.0];
		};
		
		if (!(isnil "GCamRec_ListSelectChanged")) then { _ed_listselnum = GCamRec_ListSelectChanged select 1 };
		if ( _ed_listselnum != -1 ) then
		{
			_ed_scenenum = _ed_listselnum;
			_ed_recdata = _recdata_array select _ed_listselnum;
			
			if (_ed_itemadded) then
			{
				_ed_framenum = (count _ed_recdata) - 1;
				
				_ed_itemadded = false;
			}
			else
			{
				_ed_framenum = 0;
			};
			
			_campos_rel = [ (_ed_recdata select _ed_framenum) select 0, (_ed_recdata select _ed_framenum) select 1, (_ed_recdata select _ed_framenum) select 2 ];
			_dir = (_ed_recdata select _ed_framenum) select 3;
			_dive = (_ed_recdata select _ed_framenum) select 4;
			_zoom = (_ed_recdata select _ed_framenum) select 5;
			_cam camSetFov _zoom;
			_cfzoom = sin ((_zoom / 1.8) * 90);
			
			(findDisplay 5200 displayCtrl 15) ctrlSetText ((_recdatalabel_array select _ed_scenenum) select 0);
			call _SeekbarRefresh;
			
			GCamRec_ListSelectChanged set [1, -1];
		};
		
		if (_ed_play or _ed_playall) then
		{
			_accx = 0.0;
			_accy = 0.0;
			_accz = 0.0;
			_accdir = 0.0;
			_accdive = 0.0;
			_acczoom = 0.0;
			
			_campos_rel = [ (_ed_recdata select _ed_framenum) select 0, (_ed_recdata select _ed_framenum) select 1, (_ed_recdata select _ed_framenum) select 2 ];
			_dir = (_ed_recdata select _ed_framenum) select 3;
			_dive = (_ed_recdata select _ed_framenum) select 4;
			_zoom = (_ed_recdata select _ed_framenum) select 5;
			_cam camSetFov _zoom;
			_cfzoom = sin ((_zoom / 1.8) * 90);
			
			(findDisplay 5200 displayCtrl 43) ctrlSetText format["F%1",_ed_framenum + 1];
			sliderSetPosition [50, _ed_reso * _ed_framenum];
			
			if (_ed_play) then
			{
				if (_ed_framenum == ((count _ed_recdata)-1)) then
				{
					_ed_framenum = _ed_framenum - 1;
					
					_ed_play = false;
					(findDisplay 5200 displayCtrl 21) ctrlSetTextColor COLORBUTTONTEXTOFF;
				};
			};
			if (_ed_playall) then
			{
				if (_ed_scenenum == ((count _recdata_array)-1)) then
				{
					if (_ed_framenum == ((count _ed_recdata)-1)) then
					{
						_ed_framenum = _ed_framenum - 1;
						
						_ed_playall = false;
						(findDisplay 5200 displayCtrl 22) ctrlSetTextColor COLORBUTTONTEXTOFF;
					};
				}
				else
				{
					if (_ed_framenum == ((count _ed_recdata)-1)) then
					{
						lbSetCurSel [10, _ed_scenenum + 1];
					};
				};
			};
			
			_ed_framenum = _ed_framenum + 1;
		};
		
		if ( _ed_scenenum != -1 ) then
		{
			if (!(isnil "GCamRec_SliderMove")) then { _ed_sliderpos = GCamRec_SliderMove select 1 };
			if ( _ed_sliderpos != -1 and (_ed_sliderpos != _ed_sliderpos_last)) then
			{
				if ( _ed_sliderpos >= (_ed_reso * (_ed_framenum + 1)) or _ed_sliderpos < (_ed_reso * _ed_framenum) ) then
				{
					if (_isframes) then { _ed_framenum = floor(_ed_sliderpos/_ed_reso) };
					
					_campos_rel = [ (_ed_recdata select _ed_framenum) select 0, (_ed_recdata select _ed_framenum) select 1, (_ed_recdata select _ed_framenum) select 2 ];
					_dir = (_ed_recdata select _ed_framenum) select 3;
					_dive = (_ed_recdata select _ed_framenum) select 4;
					_zoom = (_ed_recdata select _ed_framenum) select 5;
					_cam camSetFov _zoom;
					_cfzoom = sin ((_zoom / 1.8) * 90);
					
					(findDisplay 5200 displayCtrl 43) ctrlSetText format["F%1",_ed_framenum + 1];
				};
				
				_ed_sliderpos_last = _ed_sliderpos;
				GCamRec_SliderMove set [1, -1];
			};
		};
		
		if (!(isnil "GCamRec_ButtonPushedId")) then { _ed_button_id = GCamRec_ButtonPushedId };
		switch (_ed_button_id) do
		{
			case ( "rename" ):
			{
				titletext["","plain",0.0];
				
				if (_ed_scenenum != -1) then
				{
					call _HistoryAdd;
					
					call _Rename;
					call _ListRefresh;
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "rec" ):
			{
				titletext["","plain",0.0];
				
				_rec = !(_rec);
				
				if (_rec) then
				{
					(findDisplay 5200 displayCtrl 20) ctrlSetTextColor COLORBUTTONTEXTON;
					
					_recdata_prv = [];
					_ed_tickstart = diag_ticktime;
					
					titletext["Recording Started","plain",0.15];
				}
				else
				{
					_accx = 0;
					_accy = 0;
					_accz = 0;
					_accdir = 0;
					_accdive = 0;
					
					call _HistoryAdd;
					
					_recdata_array = _recdata_array + [_recdata_prv];
					_recdatalabel_array = _recdatalabel_array + [[ format["Scene %1",_reccnt], count _recdata_prv, (diag_ticktime - _ed_tickstart) ]];
					_reccnt = _reccnt + 1;
					
					(findDisplay 5200 displayCtrl 20) ctrlSetTextColor COLORBUTTONTEXTOFF;
					call _ListRefresh;
					lbSetCurSel [10, (count _recdata_array) - 1];
					_ed_itemadded = true;
					
					titletext["Recording Finished","plain",0.3];
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "play" ):
			{
				titletext["","plain",0.0];
				
				if (count _recdata_array > 0) then
				{
					_ed_play = !(_ed_play);
					if (_ed_play) then
					{
						if ( _ed_scenenum == -1 ) then { lbSetCurSel [10, 0] };
						if ( _ed_framenum == ((count _ed_recdata)-1) ) then { _ed_framenum = 0 };
						
						(findDisplay 5200 displayCtrl 21) ctrlSetTextColor COLORBUTTONTEXTON;
						
						_ed_playall = false;
						(findDisplay 5200 displayCtrl 22) ctrlSetTextColor COLORBUTTONTEXTOFF;
					}
					else
					{
						(findDisplay 5200 displayCtrl 21) ctrlSetTextColor COLORBUTTONTEXTOFF;
					};
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "playall" ):
			{
				titletext["","plain",0.0];
				
				if (count _recdata_array > 0) then
				{
					_ed_playall = !(_ed_playall);
					if (_ed_playall) then
					{
						if ( _ed_scenenum == -1 ) then { lbSetCurSel [10, 0] };
						if ( _ed_scenenum == ((count _recdata_array)-1) and _ed_framenum == ((count _ed_recdata)-1) ) then { lbSetCurSel [10, 0] };
						
						(findDisplay 5200 displayCtrl 22) ctrlSetTextColor COLORBUTTONTEXTON;
						
						_ed_play = false;
						(findDisplay 5200 displayCtrl 21) ctrlSetTextColor COLORBUTTONTEXTOFF;
					}
					else
					{
						(findDisplay 5200 displayCtrl 22) ctrlSetTextColor COLORBUTTONTEXTOFF;
					};
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "up" ):
			{
				titletext["","plain",0.0];
				
				if (_ed_scenenum > 0) then
				{
					call _HistoryAdd;
					
					call _SceneUp;
					call _ListRefresh;
					
					lbSetCurSel [10, _ed_scenenum];
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "down" ):
			{
				titletext["","plain",0.0];
				
				if ( _ed_scenenum >= 0 and _ed_scenenum < ((count _recdata_array)-1) ) then
				{
					call _HistoryAdd;
					
					call _SceneDown;
					call _ListRefresh;
					
					lbSetCurSel [10, _ed_scenenum];
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "delete" ):
			{
				titletext["","plain",0.0];
				
				if (_ed_scenenum != -1) then
				{
					call _StopPlaying;
					call _PosStopRefresh;
					
					call _HistoryAdd;
					
					call _SceneDelete;
					call _ListRefresh;
					
					if ( _ed_scenenum != -1 ) then
					{
						lbSetCurSel [10, _ed_scenenum];
					}
					else
					{
						sliderSetspeed [50, 1.0, 0.0];
						sliderSetPosition [50, 0.0];
						(findDisplay 5200 displayCtrl 41) ctrlSetText "";
						(findDisplay 5200 displayCtrl 42) ctrlSetText "";
						(findDisplay 5200 displayCtrl 43) ctrlSetText "";
						(findDisplay 5200 displayCtrl 44) ctrlSetText "";
					};
					
					titletext["Selected scene deleted","plain",0.2];
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "copy" ):
			{
				titletext["","plain",0.0];
				if (_ed_scenenum != -1) then
				{
					_isallcopy = false;
					
					(findDisplay 5200 displayCtrl 26) ctrlSetTextColor COLORBUTTONTEXTON;
					call _Convert;
					(findDisplay 5200 displayCtrl 26) ctrlSetTextColor COLORBUTTONTEXTOFF;
					
					titletext["Selected scene copied to the Clipboard","plain",0.2];
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "copyall" ):
			{
				titletext["","plain",0.0];
				if (count _recdata_array > 0) then
				{
					_isallcopy = true;
					
					(findDisplay 5200 displayCtrl 27) ctrlSetTextColor COLORBUTTONTEXTON;
					call _Convert;
					(findDisplay 5200 displayCtrl 27) ctrlSetTextColor COLORBUTTONTEXTOFF;
					
					titletext["All scenes copied to the Clipboard","plain",0.2];
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "save" ):
			{
				titletext["","plain",0.0];
				if (count _recdata_array > 0) then
				{
					call _StopPlaying;
					
					_accx = 0.0;
					_accy = 0.0;
					_accz = 0.0;
					_accdir = 0.0;
					_accdive = 0.0;
					_acczoom = 0.0;
					
					(findDisplay 5200 displayCtrl 28) ctrlSetTextColor COLORBUTTONTEXTON;
					call _Save;
					(findDisplay 5200 displayCtrl 28) ctrlSetTextColor COLORBUTTONTEXTOFF;
					titletext["All scenes copied to the Clipboard as a save data\nPaste to ""gcam_rec_savedata.hpp""","plain",1.0];
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "close" ):
			{
				 call _CloseEditor;
				
				GCamRec_ButtonPushedId = "";
			};
			
			case ( "sels" ):
			{
				titletext["","plain",0.0];
				
				if (_ed_scenenum != -1) then
				{
					call _PosStopRefresh;
					
					_ed_framenum_sels = _ed_framenum;
					(findDisplay 5200 displayCtrl 41) ctrlSetText format["F%1",_ed_framenum_sels + 1];
					
					if ( _ed_framenum_sele < _ed_framenum_sels ) then
					{
						_ed_framenum_sele = -1;
						(findDisplay 5200 displayCtrl 42) ctrlSetText "";
					};
					titletext["First frame selected","plain",0.2];
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "sele" ):
			{
				titletext["","plain",0.0];
				
				if (_ed_scenenum != -1) then
				{
					call _PosStopRefresh;
					
					_ed_framenum_sele = _ed_framenum;
					(findDisplay 5200 displayCtrl 42) ctrlSetText format["F%1",_ed_framenum_sele + 1];
					
					if ( _ed_framenum_sels > _ed_framenum_sele ) then
					{
						_ed_framenum_sels = -1;
						(findDisplay 5200 displayCtrl 41) ctrlSetText "";
					};
					titletext["Last frame selected","plain",0.2];
				};
				
				GCamRec_ButtonPushedId = "";
			};
			case ( "cut" ):
			{
				titletext["","plain",0.0];
				
				if ( _ed_framenum_sels != -1 or _ed_framenum_sele != -1 ) then
				{
					(findDisplay 5200 displayCtrl 62) ctrlSetTextColor COLORBUTTONTEXTON;
					call _StopPlaying;
					call _PosStopRefresh;
					
					call _HistoryAdd;
					
					if (_ed_framenum_sels == -1) then { _ed_framenum_sels = 0 };
					if (_ed_framenum_sele == -1) then { _ed_framenum_sele = (count _ed_recdata)-1 };
					if ( ((_ed_framenum_sele+1) - _ed_framenum_sels) == (count _ed_recdata) ) then
					{
						call _SceneDelete;
						call _ListRefresh;
						
						if ( _ed_scenenum != -1 ) then { lbSetCurSel [10, _ed_scenenum] };
					}
					else
					{
						call _SceneEditCut;
						call _SeekbarRefresh;
						call _ListRefresh;
					};
					titletext["Selected frames cutted","plain",0.2];
					(findDisplay 5200 displayCtrl 62) ctrlSetTextColor COLORBUTTONTEXTOFF;
				};
				GCamRec_ButtonPushedId = "";
			};
			case ( "new" ):
			{
				titletext["","plain",0.0];
				
				if ( _ed_framenum_sels != -1 or _ed_framenum_sele != -1 ) then
				{
					(findDisplay 5200 displayCtrl 63) ctrlSetTextColor COLORBUTTONTEXTON;
					call _StopPlaying;
					call _PosStopRefresh;
					
					call _HistoryAdd;
					
					if (_ed_framenum_sels == -1) then { _ed_framenum_sels = 0 };
					if (_ed_framenum_sele == -1) then { _ed_framenum_sele = (count _ed_recdata)-1 };
					if ( _ed_framenum_sels == 0 and _ed_framenum_sele == ((count _ed_recdata)-1) ) then
					{
						_recdata_array = _recdata_array + [(_recdata_array select _ed_scenenum)];
						_recdatalabel_array = _recdatalabel_array + [[format["Scene %1",_reccnt],(_recdatalabel_array select _ed_scenenum) select 1,(_recdatalabel_array select _ed_scenenum) select 2]];
						_reccnt = _reccnt + 1;
						
						call _ListRefresh;
					}
					else
					{
						call _SceneEditNew;
						call _ListRefresh;
					};
					titletext["New scene created from selected frames","plain",0.25];
					(findDisplay 5200 displayCtrl 63) ctrlSetTextColor COLORBUTTONTEXTOFF;
				};
				GCamRec_ButtonPushedId = "";
			};
			case ( "undo" ):
			{
				titletext["","plain",0.0];
				
				if (count _ed_hst_array > 0) then
				{
					call _StopPlaying;
					call _PosStopRefresh;
					call _Undo;
					call _ListRefresh;
					call _SeekbarRefresh;
					
					lbSetCurSel [10, _ed_scenenum];
					
					titletext["Undo done","plain",0.2];
				};
				GCamRec_ButtonPushedId = "";
			};
			case ( "rename" ):
			{
				if (_ed_scenenum != -1) then
				{
					call _HistoryAdd;
					
					call _Rename;
					call _ListRefresh;
					
					ctrlSetFocus (findDisplay 5200 displayCtrl 10);
				};
				GCamRec_ButtonPushedId = "";
			};
		};
		
		if ( count(toArray(ctrlText 15)) > 15 ) then
		{
			_ed_text = toArray(ctrlText 15);
			_ed_text set [(count _ed_text)-1,"nil"];
			_ed_text = _ed_text - ["nil"];
			(findDisplay 5200 displayCtrl 15) ctrlSetText (toString _ed_text);
		};
		
		if (!(ctrlshown(findDisplay 5200 displayCtrl 10))) then { _editorshow = false };
	};
	
	_campos_rel = [ (_campos_rel select 0) + _accx, (_campos_rel select 1) + _accy, (_campos_rel select 2) + _accz ];
	_objpos = getposASL _obj;
	_campos = [ (_objpos select 0) + (_campos_rel select 0), (_objpos select 1) + (_campos_rel select 1), (_objpos select 2) + (_campos_rel select 2) ];
	_cam setPosASL [ _campos select 0, _campos select 1, _campos select 2 ];
	_cam camSetTarget [ (_campos select 0) + (cos _dir) * (cos _dive) * 100000, (_campos select 1) + (sin _dir) * (cos _dive) * 100000, (_campos select 2) + (sin _dive) * 100000];
	_cam camCommit 0;
	
	if ( (getposATL _cam select 2) < 0.5 ) then
	{
		_cam setPosATL [ _campos select 0, _campos select 1, 0.501 ];
		_campos_rel set [ 2, (getposASL _cam select 2) - (getposASL _obj select 2) ];
		if ( _accz < 0 ) then { _accz = 0 };
	};
	
	sleep INTERVAL;
};
